/*
Module : StartX.cpp
Purpose: Implementation of a simple CreateProcess wrapper app
Created: PJN / 29-01-2003
History: PJN / 20-08-2004 1. Now returns the error code back to the OS as returned from any failures when calling 
                          the Win32 SDK calls which StartX uses.
                          2. Now includes a /NOUI command line parameter, which suppresses any messages boxes which 
                          can appear when an error occurs. Thanks to Trevor Hart for these 2 suggestions.
         PJN / 20-12-2004 1. Now supports a /RETURNERROR command line parameter, which returns the error code of
                          the process being run. Setting this parameter implies using /WAIT. Thanks to 
                          "ITAKA webmaster" for suggesting this addition.
         PJN / 19-08-2005 1. Error codes are now reported as a string by calling FormatMessage. Thanks to 
                          Dennis Bareis for prompting this update.
                          2. String resources which are displayed to the end user now use the format IDP_...


Copyright (c) 2003 - 2005 by PJ Naughter.  (Web: www.naughter.com, Email: pjna@naughter.com)

All rights reserved.

Copyright / Usage Details:

You are allowed to include the source code in any product (commercial, shareware, freeware or otherwise) 
when your product is released in binary form. You are allowed to modify the source code in any way you want 
except you cannot modify the copyright details at the top of each module. If you want to distribute source 
code with your application, then you are only allowed to distribute versions released by the author. This is 
to maintain a single distribution point for the source code. 

*/



////////////////////////// Includes ///////////////////////////////////////////

#include "stdafx.h"
#include "StartX.h"
#include "ParametersPromptDlg.h"



////////////////////////// Macros / Locals ////////////////////////////////////

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif


CStartXApp theApp;

#ifndef BELOW_NORMAL_PRIORITY_CLASS
#define BELOW_NORMAL_PRIORITY_CLASS       0x00004000
#endif

#ifndef ABOVE_NORMAL_PRIORITY_CLASS
#define ABOVE_NORMAL_PRIORITY_CLASS       0x00008000
#endif

typedef BOOL (WINAPI CREATEPROCESSWITHLOGONW)(LPCWSTR, LPCWSTR, LPCWSTR, DWORD, LPCWSTR, LPWSTR, DWORD, LPVOID, LPWSTR, LPSTARTUPINFOW, LPPROCESS_INFORMATION);
typedef CREATEPROCESSWITHLOGONW* LPCREATEPROCESSWITHLOGONW;

typedef DWORD (WINAPI CREDUIPROMPTFORCREDENTIALS)(PCREDUI_INFO, PCTSTR, PCtxtHandle, DWORD, PCTSTR, ULONG, PCTSTR, ULONG, PBOOL, DWORD);
typedef CREDUIPROMPTFORCREDENTIALS* LPCREDUIPROMPTFORCREDENTIALS;

typedef HRESULT (WINAPI SH_GET_FOLDERPATH)(HWND, int, HANDLE, DWORD, LPTSTR);
typedef SH_GET_FOLDERPATH* LPSH_GET_FOLDERPATH;

#ifndef LOGIN_WITH_PROFILE
#define LOGON_WITH_PROFILE              0x00000001
#endif

#ifndef LOGIN_NETCREDENTIALS_ONLY
#define LOGON_NETCREDENTIALS_ONLY       0x00000002
#endif

#ifndef SHGFP_TYPE_CURRENT
#define SHGFP_TYPE_DEFAULT 1
#endif

#ifndef SHGFP_TYPE_DEFAULT
#define SHGFP_TYPE_DEFAULT 1
#endif



////////////////////////// Implementation //////////////////////////////////////////

class CMyCommandLineInfo : public CCommandLineInfo
{
public:
//Constructors / Destructors
  CMyCommandLineInfo();

//Methods
  void FixStrings(CString& sValue);
  void ParseParam(LPCTSTR lpszParam, BOOL bFlag, BOOL bLast);
  CString MapFromGenericPath(const CString& sPath);
  CString GetIdlPath(int nIdl);

//Data
  CString sUser;             //The user to run the request as
  CString sDomain;           //The domain of the user to run the request as
  BOOL    bPromptPassword;   //Should we prompt for the users password
  CString sPassword;         //The password for "sUser" to login with
  CString sCommandLine;      //The command line to actually run
  DWORD   dwCreationFlags;   //The creation flags to pass to "CreateProcess.."
  CString sDesktop;          //The "lpDesktop" attribute in the STARTUPINFO structure
  BOOL    bDesktop;          //TRUE if the "lpDesktop" attribute should be used
  CString sConsoleTitle;     //The "lpTitle" attribute in the STARTUPINFO structure
  CPoint  WindowPoint;       //The x and y values of the "dwX" and "dwY" attributes in the STARTUPINFO structure
  CSize   WindowSize;        //The width and height values of the "dwXSize" and "dwYSize" attributes in the STARTUPINFO structure
  CSize   ConsoleBufferSize; //The width and height values of the "dwXCountChars" and "dwYCountChars" attributes in the STARTUPINFO structure
  DWORD   dwFillAttribute;   //The "dwFillAttribute" attribute in the STARTUPINFO structure
  DWORD   dwFlags;           //The "dwFlags" attribute in the STARTUPINFO structure
  WORD    wShowWindow;       //The "wShowWindow" attribute in the STARTUPINFO structure
  CString sCurrentDirectory; //The "lpCurrentDirectory" attribute to pass to "CreateProcess.."
  DWORD   dwLogonFlags;      //The "dwLogonFlags" attribute to pass to "CreateProcessWithLogonW"
  BOOL    bWait;             //Should we wait for the process to terminate before this app exits
  CString sDrive;            //The value which is replaced in all occurrences of "@EXEDRV@"
  CString sFolder;           //The value which is replaced in all occurrences of "@EXEDIR@"
  BOOL    bNoUI;             //Supress any GUI
  BOOL    bReturnError;      //Returns error code in process being created
};

CMyCommandLineInfo::CMyCommandLineInfo()
{
  bPromptPassword = FALSE;
  dwCreationFlags = 0;
  bDesktop = bDesktop;
  WindowPoint = CPoint(0, 0);
  WindowSize = CSize(0, 0);
  ConsoleBufferSize = CSize(0, 0);
  dwFillAttribute = 0;
  dwFlags = 0;
  wShowWindow = SW_SHOW;
  dwLogonFlags = 0;
  bWait = FALSE;
  bNoUI = FALSE;
  bReturnError = FALSE;
}

void CMyCommandLineInfo::FixStrings(CString& sValue)
{
  //Replace any variables with their actual values as determined above
  sValue.Replace(_T("@EXEDRV@"), sDrive);
  sValue.Replace(_T("@EXEDIR@"), sFolder);
  sValue.Replace(_T("@QUOTE@"), _T("\""));
  sValue = MapFromGenericPath(sValue);
}

CString CMyCommandLineInfo::MapFromGenericPath(const CString& sPath)
{
	CString sExpanded(sPath);
  CString sIdl;
  int nQuote1 = sPath.Find(_T('@'));
	if (nQuote1 != -1)
  {
    CString sGeneric(sPath.Right(sPath.GetLength() - nQuote1 - 1));
    int nQuote2 = sGeneric.Find(_T('@'));
    if (nQuote2 > 0)
    {
      sIdl = sGeneric.Left(nQuote2);
      int nIdl = _ttoi(sIdl);
      sExpanded = sPath.Left(nQuote1) + GetIdlPath(nIdl) + sPath.Right(sPath.GetLength() - nQuote2 - nQuote1 - 2);
    }
  }

  return sExpanded;
}

CString CMyCommandLineInfo::GetIdlPath(int nIdl)
{
  //What will be the return value
  CString sIdl;

  //Use the SHGetFolderPath API
  HINSTANCE hshfolder = LoadLibrary(_T("SHFOLDER.DLL"));
  if (hshfolder)
  {
    #ifdef _UNICODE
    LPSH_GET_FOLDERPATH lpfnShGetFolderPath = (LPSH_GET_FOLDERPATH) GetProcAddress(hshfolder, "SHGetFolderPathW");
    #else    
		LPSH_GET_FOLDERPATH lpfnShGetFolderPath = (LPSH_GET_FOLDERPATH) GetProcAddress(hshfolder, "SHGetFolderPathA");
    #endif
    if (lpfnShGetFolderPath)
    {
      TCHAR pszPath[_MAX_PATH];
			if (SUCCEEDED(lpfnShGetFolderPath(NULL, nIdl, NULL, SHGFP_TYPE_DEFAULT, pszPath)))
        sIdl = pszPath;
    }
    FreeLibrary(hshfolder);
  }

  return sIdl;
}

void CMyCommandLineInfo::ParseParam(LPCTSTR lpszParam, BOOL bFlag, BOOL bLast)
{
  CString sUpCaseParam(lpszParam);
  sUpCaseParam.MakeUpper();
  int nLength = sUpCaseParam.GetLength();

  if (!bFlag)
  { 
    if (!bLast)
    {
      sConsoleTitle = lpszParam;
      FixStrings(sConsoleTitle);
    }
    else
    {
      sCommandLine = lpszParam;
      FixStrings(sCommandLine);
    }
  }
  else 
  {
    if (nLength && sUpCaseParam.GetAt(0) == _T('D'))
    {
      sCurrentDirectory = &lpszParam[1];
      FixStrings(sCurrentDirectory);
    }
    else if (sUpCaseParam == _T("B"))
    {
      dwCreationFlags |= CREATE_NO_WINDOW;
    }
    else if (sUpCaseParam == _T("NOUI"))
    {
      bNoUI = TRUE;
    }
    else if (sUpCaseParam == _T("RETURNERROR"))
    {
      bReturnError = TRUE;
      bWait = TRUE;
    }
    else if (sUpCaseParam == _T("PP"))
    {
      bPromptPassword = TRUE;
    }
    else if (sUpCaseParam == _T("MIN"))
    {
      dwFlags |= STARTF_USESHOWWINDOW;
      wShowWindow = SW_SHOWMINIMIZED;
    }
    else if (sUpCaseParam == _T("MAX"))
    {
      dwFlags |= STARTF_USESHOWWINDOW;
      wShowWindow = SW_SHOWMAXIMIZED;
    }
    else if (sUpCaseParam == _T("SEPARATE"))
    {
      dwCreationFlags |= CREATE_SEPARATE_WOW_VDM;
    }
    else if (sUpCaseParam == _T("SHARED"))
    {
      dwCreationFlags |= CREATE_SHARED_WOW_VDM;
    }
    else if (sUpCaseParam == _T("LOW"))
    {
      dwCreationFlags |= IDLE_PRIORITY_CLASS;
    }
    else if (sUpCaseParam == _T("NORMAL"))
    {
      dwCreationFlags |= NORMAL_PRIORITY_CLASS;
    }
    else if (sUpCaseParam == _T("HIGH"))
    {
      dwCreationFlags |= HIGH_PRIORITY_CLASS;
    }
    else if (sUpCaseParam == _T("ABOVENORMAL"))
    {
      dwCreationFlags |= ABOVE_NORMAL_PRIORITY_CLASS;
    }
    else if (sUpCaseParam == _T("BELOWNORMAL"))
    {
      dwCreationFlags |= BELOW_NORMAL_PRIORITY_CLASS;
    }
    else if (sUpCaseParam == _T("WAIT"))
    {
      bWait = TRUE;
    }
    else if ((sUpCaseParam.Find(_T("U")) == 0) && (nLength > 1))
    {
      sUser = &lpszParam[1];
    } 
    else if ((sUpCaseParam.Find(_T("W")) == 0) && (nLength > 1))
    {
      sPassword = &lpszParam[1];
    } 
    else if ((sUpCaseParam.Find(_T("O")) == 0) && (nLength > 1))
    {
      sDomain = &lpszParam[1];
    } 
    else if ((sUpCaseParam.Find(_T("K")) == 0))
    {
      bDesktop = TRUE;
      if (nLength > 1)
        sDesktop = &lpszParam[1];
    } 
    else if ((sUpCaseParam.Find(_T("P")) == 0) && (nLength > 3))
    {
      dwFlags |= STARTF_USEPOSITION;
      CString sPoint(sUpCaseParam.Right(nLength - 1));
      _stscanf(sPoint, _T("%d,%d"), &WindowPoint.x, &WindowPoint.y);
    } 
    else if ((sUpCaseParam.Find(_T("S")) == 0) && (nLength > 3))
    {
      dwFlags |= STARTF_USESIZE;
      CString sSize(sUpCaseParam.Right(nLength - 1));
      _stscanf(sSize, _T("%d,%d"), &WindowSize.cx, &WindowSize.cy);
    } 
    else if ((sUpCaseParam.Find(_T("CS")) == 0) && (nLength > 4))
    {
      dwFlags |= STARTF_USECOUNTCHARS;
      CString sConsoleSize(sUpCaseParam.Right(nLength - 2));
      _stscanf(sConsoleSize, _T("%d,%d"), &ConsoleBufferSize.cx, &ConsoleBufferSize.cy);
    } 
    else if (sUpCaseParam == _T("FR"))
    {
      dwFlags |= STARTF_USEFILLATTRIBUTE;
      dwFillAttribute |= FOREGROUND_RED;
    }
    else if (sUpCaseParam == _T("FG"))
    {
      dwFlags |= STARTF_USEFILLATTRIBUTE;
      dwFillAttribute |= FOREGROUND_GREEN;
    }
    else if (sUpCaseParam == _T("FB"))
    {
      dwFlags |= STARTF_USEFILLATTRIBUTE;
      dwFillAttribute |= FOREGROUND_BLUE;
    }
    else if (sUpCaseParam == _T("BR"))
    {
      dwFlags |= STARTF_USEFILLATTRIBUTE;
      dwFillAttribute |= BACKGROUND_RED;
    }
    else if (sUpCaseParam == _T("BG"))
    {
      dwFlags |= STARTF_USEFILLATTRIBUTE;
      dwFillAttribute |= BACKGROUND_GREEN;
    }
    else if (sUpCaseParam == _T("BB"))
    {
      dwFlags |= STARTF_USEFILLATTRIBUTE;
      dwFillAttribute |= BACKGROUND_BLUE;
    }
    else if (sUpCaseParam == _T("FS"))
    {
      dwFlags |= STARTF_RUNFULLSCREEN;
    }
    else if (sUpCaseParam == _T("LP"))
    {
      dwLogonFlags = LOGON_WITH_PROFILE;
    }
    else if (sUpCaseParam == _T("LNCO"))
    {
      dwLogonFlags = LOGON_NETCREDENTIALS_ONLY;
    }
  }
}


BEGIN_MESSAGE_MAP(CStartXApp, CWinApp)
	//{{AFX_MSG_MAP(CStartXApp)
	//}}AFX_MSG
END_MESSAGE_MAP()

CStartXApp::CStartXApp() : m_nReturnCode(ERROR_SUCCESS)
{
}

CString CStartXApp::GetErrorDescription(DWORD dwError)
{
  //What will be the return value from this function
  CString sError;

	LPTSTR lpBuffer;
	if (FormatMessage(FORMAT_MESSAGE_ALLOCATE_BUFFER | FORMAT_MESSAGE_FROM_SYSTEM,
			              NULL, dwError, MAKELANGID(LANG_NEUTRAL, SUBLANG_SYS_DEFAULT),
			              (LPTSTR) &lpBuffer, 0, NULL))
	{
    sError = lpBuffer;
		LocalFree(lpBuffer);
	}

	return sError;
}

BOOL CStartXApp::InitInstance()
{
  USES_CONVERSION;

  //Find the current Diretory and Drive for this instance
  TCHAR pszPath[_MAX_PATH];
  GetModuleFileName(NULL, pszPath, _MAX_PATH);
  TCHAR pszDrive[_MAX_DRIVE];
  TCHAR pszDir[_MAX_DIR];
  TCHAR pszFolder[_MAX_PATH];
  _tsplitpath(pszPath, pszDrive, pszDir, NULL, NULL);
  _tmakepath(pszFolder, pszDrive, pszDir, NULL, NULL);
  CString sFolder(pszFolder);
  int nLength = sFolder.GetLength();
  if (nLength && sFolder.GetAt(nLength-1) == _T('\\'))
    sFolder = sFolder.Left(nLength-1);
  CString sDrive(pszDrive);
  nLength = sDrive.GetLength();
  if (nLength && sDrive.GetAt(nLength-1) == _T('\\'))
    sDrive = sDrive.Left(nLength-1);

  //Parse the command line
	CMyCommandLineInfo cmdInfo;
  cmdInfo.sDrive = sDrive;
  cmdInfo.sFolder = sFolder;
	ParseCommandLine(cmdInfo);
  if (cmdInfo.sCommandLine.IsEmpty())
  {
    //Display an error message if we failed to parse the command line
    CParametersPromptDlg dlg;
    dlg.DoModal();
    return FALSE;
  }

  //Are we to call CreateProcessWithLogonW
  HMODULE hAdvapi = NULL;
  LPCREATEPROCESSWITHLOGONW lpCreateProcessWithLogonW = NULL;
  if (cmdInfo.sUser.GetLength())
  {
    hAdvapi = LoadLibrary(_T("ADVAPI32.DLL"));
    if (hAdvapi)
    {
      lpCreateProcessWithLogonW = (LPCREATEPROCESSWITHLOGONW) GetProcAddress(hAdvapi, "CreateProcessWithLogonW");
      if (lpCreateProcessWithLogonW == NULL)
      {
        FreeLibrary(hAdvapi);

        m_nReturnCode = ERROR_CALL_NOT_IMPLEMENTED;

        if (!cmdInfo.bNoUI)
          AfxMessageBox(IDP_FAIL_FIND_CREATEPROCESSWITHLOGONW);

        return FALSE;
      }
    }
    else
    {
      m_nReturnCode = GetLastError();

      if (!cmdInfo.bNoUI)
        AfxMessageBox(IDP_FAIL_FIND_CREATEPROCESSWITHLOGONW);

      return FALSE;
    }
  }  

  //Do we need to call CredUIPromptForCredentials
  HMODULE hWinCred = NULL;
  LPCREDUIPROMPTFORCREDENTIALS lpCredUIPromptForCredentials = NULL;
  if (cmdInfo.bPromptPassword)
  {
    hWinCred = LoadLibrary(_T("CREDUI.DLL"));
    if (hWinCred)
    {
      #ifdef _UNICODE
      lpCredUIPromptForCredentials = (LPCREDUIPROMPTFORCREDENTIALS) GetProcAddress(hWinCred, "CredUIPromptForCredentialsW");
      #else
      lpCredUIPromptForCredentials = (LPCREDUIPROMPTFORCREDENTIALS) GetProcAddress(hWinCred, "CredUIPromptForCredentialsA");
      #endif
      if (lpCredUIPromptForCredentials == NULL)
      {
        if (hAdvapi)
          FreeLibrary(hAdvapi);
        FreeLibrary(hWinCred);

        m_nReturnCode = ERROR_CALL_NOT_IMPLEMENTED;

        if (!cmdInfo.bNoUI)
          AfxMessageBox(IDP_FAIL_FIND_CREDUIPROMPTFORCREDENTIALS);

        return FALSE;
      }
      else
      {
        //Do the actual prompting for the user
        TCHAR pszUserName[CREDUI_MAX_USERNAME_LENGTH+1];
        _tcscpy(pszUserName, cmdInfo.sUser);
        TCHAR pszPassword[CREDUI_MAX_PASSWORD_LENGTH+1];
        _tcscpy(pszPassword, cmdInfo.sPassword);
        BOOL bSave = FALSE;

        //Form the prompts to display in the dialog
        CREDUI_INFO cuii;
        ZeroMemory(&cuii, sizeof(CREDUI_INFO));
        cuii.cbSize = sizeof(CREDUI_INFO);
        CString sMessagePrompt;

        CString sProgram(cmdInfo.sCommandLine);
        sProgram.TrimLeft();
        int nSpace = sProgram.Find(_T(" "));
        if (nSpace != -1)
          sProgram = sProgram.Left(nSpace);
        AfxFormatString1(sMessagePrompt, IDS_MESSAGE_PROMPT, sProgram);
        cuii.pszMessageText = sMessagePrompt;
        CString sMessageCaption;
        sMessageCaption.LoadString(IDS_MESSAGE_CAPTION);
        cuii.pszCaptionText = sMessageCaption;

        //Display the standard dialog to prompt for credentials
        DWORD dwSuccess = lpCredUIPromptForCredentials(&cuii, NULL, NULL, 0, pszUserName, CREDUI_MAX_USERNAME_LENGTH+1, 
                                                       pszPassword, CREDUI_MAX_PASSWORD_LENGTH+1, &bSave, 
                                                       CREDUI_FLAGS_INCORRECT_PASSWORD | CREDUI_FLAGS_DO_NOT_PERSIST | CREDUI_FLAGS_USERNAME_TARGET_CREDENTIALS);
        if (dwSuccess == NO_ERROR)
        {
          //Transfer the values back to the structure
          cmdInfo.sUser = pszUserName;
          cmdInfo.sPassword = pszPassword;
        }
        else
        {
          m_nReturnCode = dwSuccess;

          if (dwSuccess != ERROR_CANCELLED)
          {
            if (!cmdInfo.bNoUI)
            {
              CString sError;
              sError.Format(_T("%d"), dwSuccess);
              CString sMsg;
              AfxFormatString2(sMsg, IDP_FAIL_RUN_CREDUIPROMPTFORCREDENTIALS, sError, GetErrorDescription(dwSuccess));
              AfxMessageBox(sMsg);
            }
          }

          //Silently fail the call
          FreeLibrary(hWinCred);
          if (hAdvapi)
            FreeLibrary(hAdvapi);

          return FALSE;
        }
      }
    }
    else
    {
      m_nReturnCode = GetLastError();

      if (hAdvapi)
        FreeLibrary(hAdvapi);

      if (!cmdInfo.bNoUI)
        AfxMessageBox(IDP_FAIL_FIND_CREDUIPROMPTFORCREDENTIALS);
      return FALSE;
    }
  }

  //Fill in the STARTUPINFO(W) structure  
  STARTUPINFO si;
  ZeroMemory(&si, sizeof(STARTUPINFO));

  STARTUPINFOW siw;
  if (lpCreateProcessWithLogonW)
  {
    ZeroMemory(&siw, sizeof(STARTUPINFO));
    siw.cb = sizeof(STARTUPINFO);
    if (cmdInfo.bDesktop)
    {
      if (cmdInfo.sDesktop.GetLength())
        siw.lpDesktop = T2W((LPTSTR) (LPCTSTR) cmdInfo.sDesktop);
    }
    if (cmdInfo.sConsoleTitle.GetLength())
      siw.lpTitle = T2W((LPTSTR) (LPCTSTR) cmdInfo.sConsoleTitle);
    siw.dwFlags = cmdInfo.dwFlags;
    if (siw.dwFlags & STARTF_USEPOSITION)
    {
      siw.dwX = cmdInfo.WindowPoint.x;
      siw.dwY = cmdInfo.WindowPoint.y;
    }
    if (siw.dwFlags & STARTF_USESIZE)
    {
      siw.dwXSize = cmdInfo.WindowSize.cx;
      siw.dwYSize = cmdInfo.WindowSize.cy;
    }
    if (siw.dwFlags & STARTF_USECOUNTCHARS)
    { 
      siw.dwXCountChars = cmdInfo.ConsoleBufferSize.cx;
      siw.dwYCountChars = cmdInfo.ConsoleBufferSize.cy;
    }
    if (siw.dwFlags & STARTF_USEFILLATTRIBUTE)
      siw.dwFillAttribute = cmdInfo.dwFillAttribute;
    siw.wShowWindow = cmdInfo.wShowWindow;
  }
  else
  {
    si.cb = sizeof(STARTUPINFO);
    if (cmdInfo.bDesktop)
    {
      if (cmdInfo.sDesktop.GetLength())
        si.lpDesktop = cmdInfo.sDesktop.GetBuffer(cmdInfo.sDesktop.GetLength());
      else
        si.lpDesktop = _T("");
    }
    if (cmdInfo.sConsoleTitle.GetLength())
      si.lpTitle = cmdInfo.sConsoleTitle.GetBuffer(cmdInfo.sConsoleTitle.GetLength());
    si.dwFlags = cmdInfo.dwFlags;
    if (si.dwFlags & STARTF_USEPOSITION)
    {
      si.dwX = cmdInfo.WindowPoint.x;
      si.dwY = cmdInfo.WindowPoint.y;
    }
    if (si.dwFlags & STARTF_USESIZE)
    {
      si.dwXSize = cmdInfo.WindowSize.cx;
      si.dwYSize = cmdInfo.WindowSize.cy;
    }
    if (si.dwFlags & STARTF_USECOUNTCHARS)
    { 
      si.dwXCountChars = cmdInfo.ConsoleBufferSize.cx;
      si.dwYCountChars = cmdInfo.ConsoleBufferSize.cy;
    }
    if (si.dwFlags & STARTF_USEFILLATTRIBUTE)
      si.dwFillAttribute = cmdInfo.dwFillAttribute;
    si.wShowWindow = cmdInfo.wShowWindow;
  }

  //Prepare the PROCESS_INFORMATION structure
  PROCESS_INFORMATION pi;
  ZeroMemory(&pi, sizeof(PROCESS_INFORMATION));

  //Prepare the command line to run
  LPTSTR pszCommandLine = cmdInfo.sCommandLine.GetBuffer(cmdInfo.sCommandLine.GetLength());
  
  BOOL bSuccess = FALSE;
  if (lpCreateProcessWithLogonW)
  {
    LPWSTR pszUser = T2W((LPTSTR) (LPCTSTR) cmdInfo.sUser);
    LPWSTR pszDomain = NULL;
    if (cmdInfo.sDomain.GetLength())
      pszDomain = T2W((LPTSTR) (LPCTSTR) cmdInfo.sDomain);
    LPWSTR pszPassword = T2W((LPTSTR) (LPCTSTR) cmdInfo.sPassword);

    bSuccess = lpCreateProcessWithLogonW(pszUser, pszDomain, pszPassword, cmdInfo.dwLogonFlags, NULL, T2W((LPTSTR) (LPCTSTR) cmdInfo.sCommandLine), cmdInfo.dwCreationFlags, 
                                         NULL, cmdInfo.sCurrentDirectory.GetLength() ? T2W((LPTSTR) (LPCTSTR) cmdInfo.sCurrentDirectory) : NULL, &siw, &pi);
  }  
  else
    bSuccess = CreateProcess(NULL, pszCommandLine, NULL, NULL, FALSE, cmdInfo.dwCreationFlags, NULL, cmdInfo.sCurrentDirectory.GetLength() ? (LPCTSTR) cmdInfo.sCurrentDirectory : NULL,
                             &si, &pi);
  if (!bSuccess)
  {
    m_nReturnCode = GetLastError();
    if (!cmdInfo.bNoUI)
    {
      CString sError;
      sError.Format(_T("%d"), m_nReturnCode);
      CString sMsg;
      AfxFormatString2(sMsg, IDP_FAIL_RUN_CREATEPROCESS, sError, GetErrorDescription(m_nReturnCode));
      AfxMessageBox(sMsg);
    }
  }
  else
  {
    //Should we wait for the app to exit
    if (cmdInfo.bWait)
    {
      WaitForSingleObject(pi.hProcess, INFINITE);

      //Get the exit code of the process if required
      if (cmdInfo.bReturnError)
      {
        DWORD dwExitCode = ERROR_SUCCESS;
        if (GetExitCodeProcess(pi.hProcess, &dwExitCode))
          m_nReturnCode = dwExitCode;
      }
    }

    CloseHandle(pi.hProcess);
    CloseHandle(pi.hThread);
  }

  //Release the CString buffers we have being using
  cmdInfo.sCommandLine.ReleaseBuffer();
  if (si.lpDesktop)
    cmdInfo.sDesktop.ReleaseBuffer();
  if (si.lpTitle)
    cmdInfo.sConsoleTitle.ReleaseBuffer();

  //Unload the ADVAPI dll if loaded
  if (hAdvapi)
    FreeLibrary(hAdvapi);

  //Unload the WinCred dll if loaded
  if (hWinCred)
    FreeLibrary(hWinCred);

	return TRUE;
}

int CStartXApp::ExitInstance() 
{
  //Let the base class do its thing
	CWinApp::ExitInstance();

  //Return the error code we have cached
  return m_nReturnCode;
}
